precision highp float;

uniform sampler2D SrcTexture;
uniform sampler2D DstTexture;
varying vec2 TextureCoordsVarying;

uniform int BlendMode;
uniform float LayerAlpha;

vec4 blendSrc(vec4 S, vec4 D) {
    vec4 vOne = vec4(1.0);
    vec4 vZero = vec4(0.0);
    vec4 T = S;
    if (BlendMode <= 0) { //normal
        T = S + D * (1.0 - S.a);
    }
    else if (BlendMode == 2) { //screen
        T = 1.0 - (1.0 - D) * (1.0 - S);
    }
    else if (BlendMode == 3) { //overlay
        //        T = S * (1.0 - D.a) + D;

        T = 2.0 * S * D;
        if (S.r < 0.5) {
            T.r = 1.0 - 2.0 * (1.0 - S.r) * (1.0 - D.r);
        }
        if (S.g < 0.5) {
            T.g = 1.0 - 2.0 * (1.0 - S.g) * (1.0 - D.g);
        }
        if (S.b < 0.5) {
            T.b = 1.0 - 2.0 * (1.0 - S.b) * (1.0 - D.b);
        }
        if (S.a < 0.5) {
            T.a = 1.0 - 2.0 * (1.0 - S.a) * (1.0 - D.a);
        }
    }
    else if (BlendMode == 1 || BlendMode == 4) { //darken multiply
        T.a = S.a + D.a - S.a * D.a;
        T.rgb = (1.0 - D.a) * S.rgb + (1.0 - S.a) * D.rgb + min(S.rgb, D.rgb);
    }
    else if (BlendMode == 5) { //lighten
        T = max(D, S);
    }
    else if (BlendMode == 6) { //clear
        T = vZero;
    }
    else if (BlendMode == 7) { //copy
        T = S;
    }
    else if (BlendMode == 8) { //sourceIn
        T = S * D.a;
    }
    else if (BlendMode == 9) { //sourceOut
        T = S * (1.0 - D.a);
    }
    else if (BlendMode == 10) { //sourceAtop
        T = S * D.a + D * (1.0 - S.a);
    }
    else if (BlendMode == 11) { //destinationOver
        T = S * (1.0 - D.a) + D;
    }
    else if (BlendMode == 12) { //destinationIn
        T = D * S.a;
    }
    else if (BlendMode == 13) { //destinationOut
        T = D * (1.0 - S.a);
    }
    else if (BlendMode == 14) { //destinationAtop
        T = S * (1.0 - D.a) + D * S.a;
    }
    else if (BlendMode == 15) { //xor
        T = S * (1.0 - D.a) + D * (1.0 - S.a);
    }
    else if (BlendMode == 16) { //plusDarker
        T = max(vZero, S + D - 1.0);
    }
    else if (BlendMode == 17) { //plusLighter
        T = min(vOne, S + D);
    } else if (BlendMode == 18){ //add
        T = max(vZero, min(S + D, vOne));
    }

    return T;
}

void main (void) {
    vec4 srcColor = LayerAlpha * texture2D(SrcTexture, TextureCoordsVarying);
    vec4 dstColor = texture2D(DstTexture, TextureCoordsVarying);

    gl_FragColor = blendSrc(srcColor, dstColor);
}
